import subprocess
from loguru import logger


def _rclone_transfer_operation(cmd, source_path, target_path=None, flags=None):
    command = cmd.copy()
    command.append(source_path)
    if target_path is not None:
        command.append(target_path)
    if flags is not None:
        command += flags
    logger.info("run {}", command)
    result = subprocess.run(command, stdout=subprocess.DEVNULL, stderr=subprocess.PIPE, check=False)
    if result.returncode != 0:
        logger.error("{} code:{} reason:{}", source_path, result.returncode, result.stderr.decode("utf-8"))

    return result.returncode


def ls(source_path):
    command = ["rclone", "lsf", source_path]

    result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
    if result.returncode == 0:
        return sorted(result.stdout.decode("utf-8").splitlines())
    reason = result.stderr.decode("utf-8")
    if "directory not found" in reason:
        return []
    logger.error("{}", result.stderr.decode("utf-8"))
    return []


def move(source_path, target_path, flags=None):
    return _rclone_transfer_operation(["rclone", "move"], source_path, target_path, flags)


def copy(source_path, target_path, flags=None):
    return _rclone_transfer_operation(['rclone', 'copy'], source_path, target_path, flags)
